@@ -18,6 +18,7 @@
 (defstruct (block-end (:include queued-op) (:copier nil))
   (suffix nil :type (or null simple-string)))
 
+(declaim (start-block))
 (defstruct (section-start (:include queued-op)
                           (:constructor nil)
                           (:copier nil))
@@ -27,9 +28,10 @@
 (defstruct (newline (:include section-start) (:copier nil))
   (kind (missing-arg)
         :type (member :linear :fill :miser :literal :mandatory)))
-(declaim (freeze-type newline))
+(declaim (freeze-type newline)
+         (end-block))
 
-#-sb-fluid (declaim (inline index-posn posn-index posn-column))
+(declaim (inline index-posn posn-index posn-column))
 (defun index-posn (index stream)
   (declare (type index index) (type pretty-stream stream)
            (values posn))
@@ -805,15 +807,34 @@ line break."
          (new (make-pprint-dispatch-table (copy-list (pp-dispatch-entries orig))
                                           (pp-dispatch-number-matchable-p orig)
                                           (pp-dispatch-only-initial-entries orig))))
-    (replace/eql-hash-table (pp-dispatch-cons-entries new)
-                            (pp-dispatch-cons-entries orig))
+    (hash-table-replace (pp-dispatch-cons-entries new) (pp-dispatch-cons-entries orig))
     new))
 
 (defun pprint-dispatch (object &optional (table *print-pprint-dispatch*))
   (declare (type (or pprint-dispatch-table null) table))
   (let* ((table (or table *initial-pprint-dispatch-table*))
+         (possibly-matchable
+          (if (pp-dispatch-only-initial-entries table)
+              ;; Filters out SYMBOL, INSTANCE, FUNCTION, NUMBER, most arrays,
+              ;; and many other types very quickly. As horrible as the code emitted
+              ;; for this check is, it's still 3x to 4x faster than >= 6 funcalls.
+              ;; The inefficiency of the array test can be fixed by transforming
+              ;; it to something clever like
+              ;;  (and (> widetag #x80) (logbitp (ash widetag -2) #xNNNN)) ; some bit pattern
+              ;; at the source level, or in the backend.
+              ;; See !PPRINT-COLD-INIT for the possibly matchable types.
+              (typep object '(or cons
+                                 (and array (not (or string bit-vector)))
+                                 sb-impl::comma))
+              ;; We could do something similar to the POSSIBLY-MATCHABLE test
+              ;; based on tracking separately whether the user added any non-cons
+              ;; entries and cons entries. So if no non-cons entries were added,
+              ;; then the table can't match OBJECT if it is an atom and in the
+              ;; set of atoms matched by the preceding TYPEP test.
+              (or (not (numberp object))
+                  (pp-dispatch-number-matchable-p table))))
          (entry
-          (when (or (not (numberp object)) (pp-dispatch-number-matchable-p table))
+          (when possibly-matchable
             (let ((cons-entry
                    (and (consp object)
                         (sb-impl::gethash/eql (car object) (pp-dispatch-cons-entries table) nil))))
@@ -913,24 +934,22 @@ line break."
 ;;;; standard pretty-printing routines
 
 (defun pprint-array (stream array)
-  (cond ((and (null *print-array*) (null *print-readably*))
-         (output-ugly-object stream array))
+  (cond ((or (null (array-element-type array))
+             (and (null *print-array*) (null *print-readably*)))
+         (print-object array stream))
         ((and *print-readably*
               (not (array-readably-printable-p array)))
          (sb-impl::output-unreadable-array-readably array stream))
         ((vectorp array)
-         (pprint-vector stream array))
+         (pprint-logical-block (stream nil :prefix "#(" :suffix ")")
+           (dotimes (i (length array))
+             (unless (zerop i)
+               (format stream " ~:_"))
+             (pprint-pop)
+             (output-object (aref array i) stream))))
         (t
          (pprint-multi-dim-array stream array))))
 
-(defun pprint-vector (stream vector)
-  (pprint-logical-block (stream nil :prefix "#(" :suffix ")")
-    (dotimes (i (length vector))
-      (unless (zerop i)
-        (format stream " ~:_"))
-      (pprint-pop)
-      (output-object (aref vector i) stream))))
-
 (defun pprint-multi-dim-array (stream array)
   (funcall (formatter "#~DA") stream (array-rank array))
   (with-array-data ((data array) (start) (end))
@@ -1474,6 +1493,9 @@ line break."
   (setf *initial-pprint-dispatch-table* nil)
   (let ((*print-pprint-dispatch* (make-pprint-dispatch-table nil nil nil)))
     (/show0 "doing SET-PPRINT-DISPATCH for regular types")
+    ;; * PLEASE NOTE : If you change these definitions, then you may need to adjust
+    ;; the computation of POSSIBLY-MATCHABLE in PPRINT-DISPATCH.
+    ;;
     ;; Assign a lower priority than for the cons entries below, making
     ;; fewer type tests when dispatching.
     (set-pprint-dispatch '(and array (not (or string bit-vector))) 'pprint-array -1)
