@@ -197,10 +197,10 @@
            (unuse-package ext "CL-USER")))
 
 #+cmu
-(setq *compile-print* nil) ; too much noise, can't see the actual warnings
+(setq cl:*compile-print* nil) ; too much noise, can't see the actual warnings
 #+sbcl
 (progn
-  (setq *compile-print* nil)
+  (setq cl:*compile-print* nil)
   (load "src/cold/muffler.lisp")
   ;; Let's just say we never care to see these.
   (declaim (sb-ext:muffle-conditions
@@ -243,12 +243,13 @@
              (gc (find-if (lambda (x) (member x '(:cheneygc :gencgc)))
                           target-feature-list))
              (arch (target-platform-keyword target-feature-list)))
-        #+nil
-        (let ((*readtable* *xc-readtable*)
-              (sb-xc:*features* target-feature-list))
-          (when (featurep '(:and :linux (:or :x86 :x86-64) :sb-thread
-                                 (:not (:or :sb-safepoint :sb-thruption))))
-            (push :pauseless-threadstart target-feature-list)))
+        ;; Win32 conditionally adds :sb-futex in grovel-features.sh
+        (when (featurep '(:and :sb-thread (:or :linux :freebsd)) target-feature-list)
+          (pushnew :sb-futex target-feature-list))
+        (when (featurep '(:and :sb-thread (:not :win32)) target-feature-list)
+          (push :pauseless-threadstart target-feature-list))
+        (when (featurep '(:and :sb-thread (:or :darwin :openbsd)) target-feature-list)
+          (push :os-thread-stack target-feature-list))
         (when (and (member :x86 target-feature-list)
                    (member :int4-breakpoints target-feature-list))
           ;; 0xCE is a perfectly good 32-bit instruction,
@@ -295,11 +296,15 @@
           ":SB-THREAD requires :GENCGC")
          ("(and pauseless-threadstart (not sb-thread))"
           ":PAUSELESS-THREADSTART requires :SB-THREAD")
+         ("(and sb-safepoint (not sb-thread))" ":SB-SAFEPOINT requires :SB-THREAD")
+         ("(and sb-thruption (not sb-safepoint))" ":SB-THRUPTION requires :SB-SAFEPOINT")
+         ("(and unix sb-safepoint-strictly)"
+          ":SB-SAFEPOINT-STRICTLY not supported on this platform")
          ("(and sb-thread (not (or riscv ppc ppc64 x86 x86-64 arm64)))"
           ":SB-THREAD not supported on selected architecture")
          ("(and gencgc cheneygc)"
           ":GENCGC and :CHENEYGC are incompatible")
-         ("(and cheneygc (not (or alpha arm arm64 hppa mips ppc riscv sparc)))"
+         ("(and cheneygc (not (or arm arm64 mips ppc riscv sparc)))"
           ":CHENEYGC not supported on selected architecture")
          ("(and gencgc (not (or sparc ppc ppc64 x86 x86-64 arm arm64 riscv)))"
           ":GENCGC not supported on selected architecture")
@@ -328,6 +333,8 @@
           ":IMMOBILE-CODE requires :IMMOBILE-SPACE feature")
          ("(and immobile-symbols (not immobile-space))"
           ":IMMOBILE-SYMBOLS requires :IMMOBILE-SPACE feature")
+         ("(and sb-futex (not sb-thread))"
+          "Can't enable SB-FUTEX on platforms lacking thread support")
          ;; There is still hope to make multithreading on DragonFly x86-64
          ("(and sb-thread x86 dragonfly)"
           ":SB-THREAD not supported on selected architecture")))
@@ -338,7 +345,7 @@
         (push (second test) failed-test-descriptions))))
   (when failed-test-descriptions
     (error "Feature compatibility check failed, ~S"
-           failed-test-descriptions)))
+           (reverse failed-test-descriptions))))
 
 ;;;; cold-init-related PACKAGE and SYMBOL tools
 
@@ -537,7 +544,18 @@
                        (if *compile-for-effect-only* "-scratch" "-tmp")))
 
          (compile-file (ecase mode
-                         (:host-compile #'compile-file)
+                         (:host-compile
+                          #+ccl
+                          (lambda (&rest args)
+                            ;; CCL doesn't like NOTINLINE on uknown functions
+                            (handler-bind ((ccl:compiler-warning
+                                             (lambda (c)
+                                               (when (eq (ccl::compiler-warning-warning-type c)
+                                                         :unknown-declaration-function)
+                                                 (muffle-warning c)))))
+                              (apply #'compile-file args)))
+                          #-ccl
+                          #'compile-file)
                          (:target-compile (if (find :assem flags)
                                               *target-assemble-file*
                                               *target-compile-file*))))
@@ -546,7 +564,7 @@
          (ignore-failure-p (find :ignore-failure-p flags)))
     (declare (type function compile-file))
 
-    (ensure-directories-exist obj :verbose *compile-print*) ; host's value
+    (ensure-directories-exist obj :verbose cl:*compile-print*) ; host's value
 
     ;; We're about to set about building a new object file. First, we
     ;; delete any preexisting object file in order to avoid confusing
