@@ -10,18 +10,17 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!ARM-ASM")
+(in-package "SB-ARM-ASM")
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
   (import '(conditional-opcode emit-word
             composite-immediate-instruction encodable-immediate
-            lsl lsr asr ror cpsr @) "SB!VM")
+            lsl lsr asr ror cpsr @) "SB-VM")
   ;; Imports from SB-VM into this package
-  (import '(sb!vm::nil-value sb!vm::registers sb!vm::null-tn sb!vm::null-offset
-            sb!vm::pc-tn sb!vm::pc-offset sb!vm::code-offset)))
+  (import '(sb-vm:nil-value sb-vm::registers sb-vm::null-tn sb-vm::null-offset
+            sb-vm::pc-tn sb-vm::pc-offset sb-vm::code-offset)))
 
-(defconstant +disassem-inst-alignment-bytes+ 4)
 
 
 (defconstant-eqx +conditions+
@@ -41,7 +40,7 @@
     (:le . 13)
     (:al . 14))
   #'equal)
-(defconstant-eqx sb!vm::+condition-name-vec+
+(defconstant-eqx sb-vm::+condition-name-vec+
   #.(let ((vec (make-array 16 :initial-element nil)))
       (dolist (cond +conditions+ vec)
         (when (null (aref vec (cdr cond)))
@@ -153,6 +152,33 @@
   (rn :field (byte 4 16) :type 'reg)
   (rd :field (byte 4 12) :type 'reg))
 
+;;; Not sure if we can coerce our disassembler to print anything resembling this:
+;;; <LDM|STM>{cond}<FD|ED|FA|EA|IA|IB|DA|DB> Rn{!},<Rlist>{^} where:
+;;;   {cond}  two-character condition mnemonic. See Table 4-2: Condition code
+;;;           summary on page 4-5.
+;;;   Rn      is an expression evaluating to a valid register number
+;;;   <Rlist> is a list of registers and register ranges enclosed in {} (For example,
+;;;           {R0,R2-R7,R10}).
+;;;   {!}     if present requests write-back (W=1), otherwise W=0
+;;;   {^}     if present set S bit to load the CPSR along with the PC, or force transfer
+;;;           of user bank when in privileged mode
+;;; not to mention the alternative mnemonics PUSH and POP
+;;; when using the native stack pointer as base register.
+(define-instruction-format
+    ;; This is just to show something in the disassembly other than BYTE ...
+    (ldm/stm 32 :default-printer '(:name cond :tab bits ", " rn ", " reglist))
+  (cond :field (byte 4 28) :type 'condition-code)
+  (opcode-3 :field (byte 3 25))
+  (bits :field (byte 4 21)) ; complicated
+  (opcode-l :field (byte 1 20))
+  (rn :field (byte 4 16) :type 'reg)
+  (reglist :field (byte 16 0)
+           :printer (lambda (value stream dstate)
+                      (declare (ignore dstate))
+                      (format stream "{~{R~d~^,~}}"
+                              (loop for i below 16
+                                    when (logbitp i value) collect i)))))
+
 (define-instruction-format (swi 32
                             :default-printer '(:name cond :tab "#" swi-number))
   (cond :field (byte 4 28) :type 'condition-code)
@@ -485,9 +511,9 @@
   (max 0 (1- (integer-length (logand integer-value (- integer-value))))))
 
 ;; FIXME: it would be idiomatic to use (DEFINE-INSTRUCTION-MACRO COMPOSITE ...)
-;; instead of exporting another instruction-generating macro into SB!VM.
+;; instead of exporting another instruction-generating macro into SB-VM.
 ;; An invocation would resemble (INST COMPOSITE {ADD|SUB|whatever| ARGS ...)
-(defmacro composite-immediate-instruction (op r x y &key fixnumize neg-op invert-y invert-r single-op-op first-op first-no-source)
+(defmacro composite-immediate-instruction (op r x y &key fixnumize neg-op invert-y invert-r single-op-op first-op first-no-source temporary)
   ;; Successively applies 8-bit wide chunks of Y to X using OP storing the result in R.
   ;;
   ;; If FIXNUMIZE is true, Y is fixnumized before being used.
@@ -499,17 +525,22 @@
   ;; it is used for a single operation instead of OP.
   ;; If FIRST-OP is given, it is used in the first iteration instead of OP.
   ;; If FIRST-NO-SOURCE is given, there will be ne source register (X) in the first iteration.
+  ;; If TEMPORARY is given, it should be a non-descriptor register
+  ;; used for the accumulation of a temporary non-descriptor.  Only makes sense with INVERT-R
+  (when temporary
+    (aver invert-r))
   (let ((bytespec (gensym "bytespec"))
         (value (gensym "value"))
-        (transformed (gensym "transformed")))
+        (transformed (gensym "transformed"))
+        (acc (gensym "acc")))
     (labels ((instruction (source-reg op neg-op &optional no-source)
                `(,@(if neg-op
                         `((if (< ,y 0)
-                              (inst ,neg-op ,r ,@(when (not no-source)`(,source-reg))
+                              (inst ,neg-op ,acc ,@(when (not no-source)`(,source-reg))
                                     (mask-field ,bytespec ,value))
-                              (inst ,op ,r ,@(when (not no-source) `(,source-reg))
+                              (inst ,op ,acc ,@(when (not no-source) `(,source-reg))
                                     (mask-field ,bytespec ,value))))
-                        `((inst ,op ,r ,@(when (not no-source) `(,source-reg))
+                        `((inst ,op ,acc ,@(when (not no-source) `(,source-reg))
                                 (mask-field ,bytespec ,value))))
                   (setf (ldb ,bytespec ,value) 0)))
              (composite ()
@@ -518,9 +549,9 @@
                   (do ((,bytespec (byte 8 (logandc1 1 (lowest-set-bit-index ,value)))
                                   (byte 8 (logandc1 1 (lowest-set-bit-index ,value)))))
                       ((zerop ,value))
-                    ,@(instruction r op neg-op)
-                    ,@(when invert-r
-                            `((inst mvn ,r ,r)))))))
+                    ,@(instruction acc op neg-op))
+                  ,@(when invert-r
+                      `((inst mvn ,r ,acc))))))
       `(let* ((,transformed ,(if fixnumize
                                  `(fixnumize ,y)
                                  `,y))
@@ -529,7 +560,8 @@
                                  `((if (< ,transformed 0) (- ,transformed) ,transformed))
                                  (if invert-y
                                      `((lognot ,transformed))
-                                     `(,transformed))))))
+                                   `(,transformed)))))
+              (,acc (or ,temporary ,r)))
          ,@(if single-op-op
               `((if (encodable-immediate ,transformed)
                     (inst ,single-op-op ,r ,x ,transformed)
@@ -719,11 +751,11 @@
 ;;; officially undefined instruction as a single-instruction SIGTRAP
 ;;; generation instruction, or breakpoint.
 (define-instruction debug-trap (segment)
-  (:printer debug-trap ((opcode-32 #!+linux #xe7f001f0
-                                   #!+netbsd #xe7ffdefe))
+  (:printer debug-trap ((opcode-32 #+linux #xe7f001f0
+                                   #+(or netbsd openbsd) #xe7ffdefe))
             :default :control #'debug-trap-control)
   (:emitter
-   (emit-word segment #!+linux #xe7f001f0 #!+netbsd #xe7ffdefe)))
+   (emit-word segment #+linux #xe7f001f0 #+(or netbsd openbsd) #xe7ffdefe)))
 
 ;;;; Miscellaneous arithmetic instructions
 
@@ -1045,6 +1077,11 @@
   (define-load/store-instruction str :store :word)
   (define-load/store-instruction strb :store :byte))
 
+(define-instruction ldm (segment &rest args) ; load multiple
+  (:printer ldm/stm ((opcode-3 #b100) (opcode-l 1))))
+(define-instruction stm (segment &rest args) ; store multiple
+  (:printer ldm/stm ((opcode-3 #b100) (opcode-l 0))))
+
 ;;; Emit a miscellaneous load/store instruction.  CONDITION is a
 ;;; condition code name, OPCODE1 is the low bit of the first opcode
 ;;; field, OPCODE2 is the second opcode field, DATA is a register TN
@@ -1254,7 +1291,7 @@
      ;; apply the final 12 bits with LDR.  For now, we'll allow up to 20
      ;; bits of displacement, as that should be easy to implement, and a
      ;; megabyte large code object is already a bit unwieldly.  If
-     ;; neccessary, we can expand to a 28 bit displacement.
+     ;; necessary, we can expand to a 28 bit displacement.
      (labels ((compute-delta (position &optional magic-value)
                 (- (label-position label
                                    (when magic-value position)
