@@ -146,7 +146,7 @@
   ;; 1.0.20.30 failed here.
   (checked-compile '(lambda () (foo 5 :w 10 :foo 15)))
   (assert
-   (not (sb-kernel::args-type-keyp (sb-int:proclaimed-ftype 'foo)))))
+   (not (sb-kernel::args-type-keyp (sb-int:global-ftype 'foo)))))
 
 ;; If the GF has &KEY and &ALLOW-OTHER-KEYS, the methods' keys can be
 ;; anything, and we don't warn about unrecognized keys.
@@ -160,14 +160,14 @@
   ;; Now the FTYPE is computed the instant anyone asks for it.
   (assert (equal (mapcar 'sb-kernel:key-info-name
                          (sb-kernel:fun-type-keywords
-                          (sb-int:proclaimed-ftype 'foo)))
+                          (sb-int:global-ftype 'foo)))
                  '(:y :z)))
   (checked-compile '(lambda () (foo 5 :z 10 :y 15)))
   (checked-compile '(lambda () (foo 5 :z 10 :foo 15)))
   (assert
-   (sb-kernel::args-type-keyp (sb-int:proclaimed-ftype 'foo)))
+   (sb-kernel::args-type-keyp (sb-int:global-ftype 'foo)))
   (assert
-   (sb-kernel::args-type-allowp (sb-int:proclaimed-ftype 'foo))))
+   (sb-kernel::args-type-allowp (sb-int:global-ftype 'foo))))
 
 ;; If any method has &ALLOW-OTHER-KEYS, 7.6.4 point 5 seems to say the
 ;; GF should be construed to have &ALLOW-OTHER-KEYS.
@@ -176,8 +176,8 @@
   (defgeneric foo (x &key))
   (defmethod foo ((x integer) &rest y &key &allow-other-keys) (list x y))
   (checked-compile '(lambda () (foo 10 :foo 20)))
-  (assert (sb-kernel::args-type-keyp (sb-int:proclaimed-ftype 'foo)))
-  (assert (sb-kernel::args-type-allowp (sb-int:proclaimed-ftype 'foo))))
+  (assert (sb-kernel::args-type-keyp (sb-int:global-ftype 'foo)))
+  (assert (sb-kernel::args-type-allowp (sb-int:global-ftype 'foo))))
 
 (fmakunbound 'foo)
 (with-test (:name (defmethod symbol-macrolet))
@@ -194,3 +194,82 @@
   (defmethod foo ((x t)) 3)
   (assert (= (foo t) 3))
   (assert (= (foo 3) 2)))
+
+(with-test (:name :bug-309084-a-i)
+  (assert-error (eval '(define-method-combination bug-309084-a-i :documentation :operator))
+                program-error))
+(with-test (:name :bug-309084-a-ii)
+  (assert-error (eval '(define-method-combination bug-309084-a-ii :documentation nil))
+                program-error))
+(with-test (:name :bug-309084-a-iii)
+  (assert-error (eval '(define-method-combination bug-309084-a-iii nil))
+                program-error))
+(with-test (:name :bug-309084-a-vi)
+  (assert-error (eval '(define-method-combination bug-309084-a-vi nil nil
+                        (:generic-function)))
+                program-error))
+(with-test (:name :bug-309084-a-vii)
+  (assert-error (eval '(define-method-combination bug-309084-a-vii nil nil
+                        (:generic-function bar baz)))
+                program-error))
+(with-test (:name :bug-309084-a-viii)
+  (assert-error (eval '(define-method-combination bug-309084-a-viii nil nil
+                        (:generic-function (bar))))
+                program-error))
+(with-test (:name :bug-309084-a-ix)
+  (assert-error (eval '(define-method-combination bug-309084-a-ix nil ((3))))
+                program-error))
+(with-test (:name :bug-309084-a-x)
+  (assert-error (eval '(define-method-combination bug-309084-a-x nil ((a))))
+                program-error))
+(with-test (:name :bug-309084-a-iv)
+  (assert-error (eval '(define-method-combination bug-309084-a-iv nil nil
+                        (:arguments order &aux &key)))
+                program-error))
+(with-test (:name :bug-309084-a-v)
+  (assert-error (eval '(define-method-combination bug-309084-a-v nil nil
+                        (:arguments &whole)))
+                program-error))
+
+(let (warnings)
+  (handler-bind ((warning (lambda (c) (push c warnings))))
+    (eval '(define-method-combination bug-309084-b/mc nil
+             ((all *))
+             (:arguments x &optional (y 'a yp) &key (z 'b zp) &aux (w (list y z)))
+             `(list ,x ,y ,yp ,z ,zp ,w)))
+    ;; Should not get any "assigned but never read" warnings.
+    (assert (= (length warnings) 1))
+    (assert (search "&OPTIONAL and &KEY" (princ-to-string (car warnings))))))
+
+(defgeneric bug-309084-b/gf (a &optional b &key &allow-other-keys)
+  (:method-combination bug-309084-b/mc)
+  (:method (m &optional n &key) (list m n)))
+
+(with-test (:name :bug-309084-b)
+  (assert (equal (bug-309084-b/gf 1) '(1 a nil b nil (a b))))
+  (assert (equal (bug-309084-b/gf 1 2) '(1 2 t b nil (2 b))))
+  (assert (equal (bug-309084-b/gf 1 2 :z 3) '(1 2 t 3 t (2 3)))))
+
+(defgeneric bug-309084-b/gf2 (a b &optional c d &key &allow-other-keys)
+  (:method-combination bug-309084-b/mc)
+  (:method (m n &optional o p &key) (list m n o p)))
+
+(with-test (:name :bug-309084-b2)
+  (assert (equal (bug-309084-b/gf2 1 2) '(1 a nil b nil (a b))))
+  (assert (equal (bug-309084-b/gf2 1 2 3) '(1 3 t b nil (3 b))))
+  (assert (equal (bug-309084-b/gf2 1 2 3 4) '(1 3 t b nil (3 b))))
+  (assert (equal (bug-309084-b/gf2 1 2 :z t) '(1 :z t b nil (:z b))))
+  (assert (equal (bug-309084-b/gf2 1 2 3 4 :z 5) '(1 3 t 5 t (3 5)))))
+
+(defmethod bug-1840595-a (x y))
+(defmethod bug-1840595-z (x))
+
+(with-test (:name :bug-1840595/reader)
+  (eval '(defclass bug-1840595r () ()))
+  (assert-error (eval '(defclass bug-1840595r () ((a :reader bug-1840595-a)))))
+  (eval '(defclass bug-1840595r () ())))
+
+(with-test (:name :bug-1840595/writer)
+  (eval '(defclass bug-1840595w () ()))
+  (assert-error (eval '(defclass bug-1840595w () ((z :writer bug-1840595-z)))))
+  (eval '(defclass bug-1840595w () ())))
