@@ -11,6 +11,53 @@
 
 (load "compiler-test-util.lisp")
 
+;;;; Better ensure nothing in messed up in the ID space
+;;;; or else everything is suspect.
+(defun layout-id-vector-sap (layout)
+  (sb-sys:sap+ (sb-sys:int-sap (sb-kernel:get-lisp-obj-address layout))
+               (- (ash (+ sb-vm:instance-slots-offset
+                          (sb-kernel:get-dsd-index
+                           sb-kernel:layout sb-kernel::id-word0))
+                       sb-vm:word-shift)
+                  sb-vm:instance-pointer-lowtag)))
+
+(let ((hash (make-hash-table)))
+  ;; assert that all layout IDs are unique
+  (let ((all-layouts
+         (delete-if
+          ;; temporary layouts (created for parsing DEFSTRUCT)
+          ;; must be be culled out.
+          (lambda (x)
+            (and (typep (sb-kernel:layout-classoid x)
+                        'sb-kernel:structure-classoid)
+                 (eq (sb-kernel::layout-equalp-impl x)
+                     #'sb-kernel::equalp-err)))
+          (sb-vm::list-allocated-objects :all
+                                         :type sb-vm:instance-widetag
+                                         :test #'sb-kernel::layout-p))))
+    (dolist (layout all-layouts)
+      (let ((id (sb-kernel:layout-id layout)))
+        (sb-int:awhen (gethash id hash)
+          (error "ID ~D is ~A and ~A" id sb-int:it layout))
+        (setf (gethash id hash) layout)))
+    ;; assert that all inherited ID vectors match the layout-inherits vector
+    (let ((structure-object
+           (sb-kernel:find-layout 'structure-object)))
+      (dolist (layout all-layouts)
+        (when (find structure-object (sb-kernel:layout-inherits layout))
+          (let ((ids
+                 (sb-sys:with-pinned-objects (layout)
+                   (let ((sap (layout-id-vector-sap layout)))
+                     (loop for depthoid from 2 to (sb-kernel:layout-depthoid layout)
+                           collect (sb-sys:signed-sap-ref-32 sap (ash (- depthoid 2) 2))))))
+                (expect
+                 (map 'list 'sb-kernel:layout-id
+                      (sb-kernel:layout-inherits layout))))
+            (unless (equal (append '(1 2) ids)
+                           (append expect (list (sb-kernel:layout-id layout))))
+              (error "Wrong IDs for ~A: expect ~D actual ~D~%"
+                     layout expect ids))))))))
+
 ;;;; examples from, or close to, the Common Lisp DEFSTRUCT spec
 
 ;;; Type mismatch of slot default init value isn't an error until the
@@ -34,19 +81,15 @@
 (defstruct (boa-kid (:include boa-saux)))
 (defstruct (boa-grandkid (:include boa-kid)))
 (with-test (:name :defstruct-boa-typecheck)
-  (flet ((dsd-always-boundp (dsd)
-           ;; A copy of sb-kernel::dsd-always-boundp, which may not
-           ;; survive make-target-2's shake-packages.
-           (logbitp 3 (sb-kernel::dsd-bits dsd))))
     (dolist (dsd (sb-kernel:dd-slots
                   (sb-kernel:find-defstruct-description 'boa-saux)))
       (let ((name (sb-kernel:dsd-name dsd))
-            (always-boundp (dsd-always-boundp dsd)))
+            (always-boundp (sb-kernel:dsd-always-boundp dsd)))
         (ecase name
           ((a c) (assert (not always-boundp)))
           (b (assert always-boundp)))))
     (let ((dd (sb-kernel:find-defstruct-description 'boa-grandkid)))
-      (assert (not (dsd-always-boundp (car (sb-kernel:dd-slots dd))))))
+      (assert (not (sb-kernel:dsd-always-boundp (car (sb-kernel:dd-slots dd))))))
     (let ((s (make-boa-saux)))
       (locally (declare (optimize (safety 3))
                         (inline boa-saux-a))
@@ -55,7 +98,7 @@
       (setf (boa-saux-c s) 5)
       (assert (eql (boa-saux-a s) 1))
       (assert (eql (boa-saux-b s) 3))
-      (assert (eql (boa-saux-c s) 5)))))
+      (assert (eql (boa-saux-c s) 5))))
 
 (with-test (:name :defstruct-boa-nice-error :skipped-on :interpreter)
   (let ((err (nth-value 1 (ignore-errors (boa-saux-a (make-boa-saux))))))
@@ -252,9 +295,10 @@
 
 (declaim (optimize (debug 2)))
 
+(declaim (muffle-conditions style-warning)) ; &OPTIONAL and &KEY
+
 (defmacro test-variant (defstructname &key colontype boa-constructor-p)
   `(locally
-     (declare (muffle-conditions style-warning)) ; &OPTIONAL and &KEY
      #+nil(format t "~&/beginning PROGN for COLONTYPE=~S~%" ',colontype)
      (defstruct (,defstructname
                   ,@(when colontype `((:type ,colontype)))
