@@ -27,14 +27,16 @@
 
 ;;;; utilities to support tests
 
-;;; string equality modulo deletion of TABs and SPACEs (as a crude way
+;;; string equality modulo deletion of consecutive whitespace (as a crude way
 ;;; of washing away irrelevant differences in indentation)
 (defun string-modulo-tabspace (s)
-  (remove-if (lambda (c)
-               (or (char= c #\space)
-                   (char= c #\tab)
-                   (char= c #\newline)))
-             s))
+  (let ((s (string-trim '(#\Space) (substitute #\Space #\Newline
+                                               (substitute #\Space #\Tab s)))))
+    (loop (let ((p (search "  " s)))
+            (if (not p) (return s))
+            ;; Extremely inefficient but simple algorithm.
+            (setq s (concatenate 'string (subseq s 0 p) (subseq s (1+ p))))))))
+
 (defun string=-modulo-tabspace (x y)
   (string= (string-modulo-tabspace x)
            (string-modulo-tabspace y)))
@@ -467,7 +469,9 @@ Form: (FOO A)   Context: EVAL
 Form: 'GLOBAL-FOO   Context: EVAL
 \(EVAL-WHEN (:EXECUTE :COMPILE-TOPLEVEL :LOAD-TOPLEVEL) A (FOO A))")))
 
-(test-util:with-test (:name (:walk multiple-value-bind))
+;; This test doesn't understand that gensyms appear in the expansion
+;; (We either need "string=-modulo-tabspace-and-gensyms" or a sexpr-based comparison)
+(test-util:with-test (:name (:walk multiple-value-bind) :fails-on :sbcl)
   (assert (string=-modulo-tabspace
            (with-output-to-string (*standard-output*)
              (take-it-out-for-a-test-walk (multiple-value-bind (a b)
@@ -480,7 +484,8 @@ Form: A   Context: EVAL; lexically bound
 Form: B   Context: EVAL; lexically bound
 \(MULTIPLE-VALUE-BIND (A B) (FOO A B) (LIST A B))")))
 
-(test-util:with-test (:name (:walk multiple-value-bind special))
+;; This test doesn't understand that gensyms appear in the expansion
+(test-util:with-test (:name (:walk multiple-value-bind special) :fails-on :sbcl)
   (assert (string=-modulo-tabspace
            (with-output-to-string (*standard-output*)
              (take-it-out-for-a-test-walk (multiple-value-bind (a b)
@@ -982,11 +987,9 @@ Form: 2   Context: EVAL
              (take-it-out-for-a-test-walk (cond (a b)
                                                 ((foo bar) a (foo a)))))
            "Form: (COND (A B) ((FOO BAR) A (FOO A)))   Context: EVAL
-Form: (IF A (PROGN B) (COND ((FOO BAR) A (FOO A))))   Context: EVAL
+Form: (IF A B (IF (FOO BAR) (PROGN A (FOO A)) NIL))   Context: EVAL
 Form: A   Context: EVAL
-Form: (PROGN B)   Context: EVAL
 Form: B   Context: EVAL
-Form: (COND ((FOO BAR) A (FOO A)))   Context: EVAL
 Form: (IF (FOO BAR) (PROGN A (FOO A)) NIL)   Context: EVAL
 Form: (FOO BAR)   Context: EVAL
 Form: 'GLOBAL-FOO   Context: EVAL
@@ -1056,6 +1059,15 @@ Form: C   Context: EVAL; lexically bound
 \(LET* ((A A) (B A) (C B) (B C))
   (DECLARE (SPECIAL A B))
   (LIST A B C))")))
+
+(test-util:with-test (:name (:walk defclass :type :initform))
+  ;; A slot with :TYPE and :INITFORM causes SB-C::WITH-SOURCE-FORM to
+  ;; appear in the expansion which didn't have a walker template at
+  ;; some point. We just make sure walking the form doesn't signal an
+  ;; error.
+  (with-output-to-string (*standard-output*)
+    (take-it-out-for-a-test-walk
+     (defclass foo () ((%bar :type integer :initform 'string))))))
 
 ;;;; more tests
 
